! 2D arterial flow simulation where inlet is enforced with a physical 
! pump and outlet is obtained from a 1D lumped model utilizing the method
! of characteristics. 
!
! This test case is derived from the older tutorial case
! "Blood ejection from a ventricle into aorta". 
! Here the 1D mesh is created internally while in the old 
! tutorial case it was done by modifying the mesh files directly.
!
! References for the methods used:
!
! 1) A method for partitioned fluid-structure interaction computation of flow in arteries 
! Medical Engineering & Physics, Volume 30, Issue 7, September 2008, Pages 917-923
! Esko Jarvinen, Peter Raback, Mikko Lyly, Juha-Pekka Salenius
!
! 2) Fluid-structure interaction boundary condition by artificial compressibility
! CFD Eccomas 2001, Swansea
! Peter Raback, Juha Ruokolainen, Mikko Lyly, and Esko Jarvinen
!
! 13.8.2013 / P.R.

 Check Keywords Warn

Header
  Mesh DB "." "contra"
End

Constants
  Gravity(4) = Real 0 -1 0 9.81
  Stefan Boltzmann = Real 5.67e-8
End

Simulation
  Coordinate System = Axi Symmetric
  Simulation Type =Transient
  Steady State Max Iterations = 250
  TimeStepping Method = BDF  
  BDF Order = 2              
  Timestep Sizes = OL.get(Parameters/2Elmer/TimeStep) 
  Timestep Intervals = OL.get(Parameters/2Elmer/NumberIter) 
  Output Intervals = 1
  Simulation Timing = Logical True
End


! Au "Body n" de Elmer correspond la Physical Surface(n) de Gmsh, n=1,2,3,4
! Le "Body 5" est gnr par le Solver 1, et pas par Gmsh
! A chaque "Body" correspond une "Equation" (cd une ensemble de "Solvers"
! appels pour ce Body/Physical surface), et un "Material"

!==== Bodies

Body 1
  Equation = 1
  Material = 1
End

Body 2
  Equation = 1
  Material = 1
End 

Body 3
  Equation = 1
  Material = 2
End

Body 4
  Equation = 2
  Material = 3
End

Body 5 
  Equation = 3
  Material = 4
  Body Force = 1
  Initial Condition = 1
End

Initial Condition 1
  Wnodal = Variable Coordinate
  Real Procedure "ArteryOutlet" "OutletInit"
End

Body Force 1
  Source = Real 0.0
End

!==== Materials

! Les "Materials" donnent les paramtres ncessaires  tous les solveurs
! - Density et Viscosity pour le Solver Navier-Stokes:
! - Young Modulus et Poisson Ratio pour le Solver Mesh Update
!   qui modifie  chaque pas de temps la position des noeuds 
!   du Body 1 en fonction des dplacements calculs de la paroi artrielle
!   en rsolvant un problme d lasticit fictif.

Material 1
  Name = "Blood"
  Density = 1000
  Viscosity = OL.get(Parameters/3Model/Viscosity)
  Youngs Modulus = 1  ! pour Mesh Update
  Poisson Ratio = 0.3 ! pour Mesh Update
End

Material 2
  Name = "Blood with FSI"
  Density = 1000
  Viscosity = OL.get(Parameters/3Model/Viscosity)
  Youngs Modulus = 1   ! pour Mesh Update
  Poisson Ratio = 0.3  ! pour Mesh Update
  Compressibility Model = Artificial Compressible
  Artificial Compressibility  = 2.4E-5 
End

Material 3
  Name = "Elastic Wall"
  Density = 1010
  Youngs Modulus = OL.get(Parameters/3Model/Elasticity)
  Poisson Ratio =  OL.get(Parameters/3Model/Poisson_ratio)
End

Material 4
  Name = "1D Artery"
  Density = 1010.0
  Artery Wall Youngs Modulus = Real OL.get(Parameters/3Model/Elasticity)
  Artery Radius = Real OL.get(Parameters/1Geometry/Artery_radius) 
  Artery Wall Thickness = Real OL.get(Parameters/1Geometry/Wall_thickness)
  Artery Poisson Ratio  = Real OL.get(Parameters/3Model/Poisson_ratio)
End

!==== Solvers

! A chaque itration, les solveurs sont appels dans l ordre suivant:
! 1. Navier-Stokes, 2. Elasticit, 3. Mesh Update

Solver 1
  Equation = Artery Outlet Solver
  Time Derivative Order = 1
  Stabilize = Logical False
  Variable = Wnodal
  Variable DOFs = 1
  Procedure = "ArteryOutlet" "OutletCompute"
  Linear System Solver = Direct
  Linear System Direct Method = Banded
  Linear System Iterative Method = CG
  Linear System Convergence Tolerance = 1.0e-6
  Linear System Max Iterations = 1000
  Linear System Preconditioning = ILU0
  Nonlinear System Max Iterations = Integer 1000
  Nonlinear System Convergence Tolerance = Real 1.0e-8
  Optimize Bandwidth = False

! These define the 1D mesh that is created on-the-fly for this solver
  1D Mesh Create = Logical True
  1D Element Order = Integer 1
  1D Number Of Elements = Integer 20
  1D Mesh Length = Real 0.1
  1D Active Direction = Integer 2
  1D Body Id = Integer 5 ! Le "Body 5" est gnr par ce solveur et pas par Gmsh
End

$Relax = 1.0
Solver 2
  Equation = Navier-Stokes
  Variable = Flow Solution[Velocity:2 Pressure:1]
  Stabilize = Logical True
  Linear System Solver = Direct
  Linear System Direct Method = umfpack
  Nonlinear System Max Iterations = 1
  Nonlinear System Convergence Tolerance = 1.0e-3
  Nonlinear System Newton After Tolerance = 5.0e-2   
  Nonlinear System Newton After IterationS = 0     
  Steady State Convergence Tolerance = 1.0e-4
  Nonlinear System Relaxation Factor = $Relax
  !Calculate Viscous Force = Logical True
End

Solver 3
  Equation = Elasticity Solver
  Variable = Displacement
  Variable DOFs = 2
  Procedure = "ElasticSolve" "ElasticSolver"
  Time Derivative Order = 2
  Linear System Solver = Direct
  Linear System Direct Method = umfpack
  Nonlinear System Newton After Tolerance = 5.0e-2
  Nonlinear System Newton After Iterations = 0
  Nonlinear System Max Iterations = 1
  Nonlinear System Convergence Tolerance = 1.0e-4
  Steady State Convergence Tolerance = 1.0e-4
  Nonlinear System Relaxation Factor = $Relax
  !Calculate stresses = True
End

Solver 4
  Equation = Mesh Update
  Linear System Solver = Direct
  Linear System Direct Method = umfpack
  Linear System Symmetric = True
  Steady State Convergence Tolerance = 1.0e-4
End

! Creation d un fichier avec vitesses et pression 
! evaluees en des points particuliers
Solver 5 ! ElmerModelsManuel Model 44
  Exec solver = "After timestep"
  Equation = "SaveScalars"
  Procedure = "SaveData" "SaveScalars"
  Exact Coordinates = Logical True
  Save Coordinates(8,2) = 1e-6 0.12 1e-6 0.22 1e-6 0.32 1e-6 0.42 0.0135 0.12 0.0135 0.22 0.0135 0.32 0.0135 0.42
  Filename = "output.txt"
End

Solver 6
   Exec solver = "After timestep"
   Equation = String "ResultOutput"
   Procedure = File "ResultOutputSolve" "ResultOutputSolver"
   Output File Name = File "velocity.pos"
   Output Format = String "gmsh"
   Gmsh Format = Logical true
! Par dfaut Elmer crit les champs "pressure", "displacement", "mesh velocity"
!  Scalar Field 1 = String pressure
!   Vector Field 1 = String Displacement
!   Vector Field 2 = String Mesh Velocity
   Vector Field 4 = String Flow Solution
   Vector Field 5 = String Mesh Update
End

!Solver 7 
!  Equation = SaveScalars
!  Procedure = "SaveData" "SaveScalars"
!  Exec Solver = After timestep
!  Filename = f.dat
!  Variable 1 = coupled iter 
!End  

! Cration d un fichier 
! avec les valeurs du Shear Stress sur la paroi artrielle
!Solver 8 ! ElmerModelsManuel Model 42
!  Equation = Fluidic Force
!  Procedure = "FluidicForce" "ForceCompute"
!  Calculate Viscous Force = Logical True
!  Shear Stress Output = Logical True
!  Shear Stress Output File = File "wss.txt"
!End

!====  Equations

Equation 1
  Navier-Stokes = True
  Mesh Update = True
  Result Output = Logical True
  Fluidic Force = Logical True
End

Equation 2
  Elasticity Solver = Logical True
  Result Output = Logical True
End

Equation 3
  Artery Outlet Solver = Logical True
  Result Output = Logical True
End


!==== Boundary conditions

! Direction 1 = x = horizontal
! Direction 2 = y = vertical

! Le Solver "Mesh Update" est associ  l'Equation 1
! qui est rsolue sur les Bodies 1, 2 et 3.
! Pour toute les frontires qui touchent les Bodies 1, 2, 3
! il faut donc aussi spcifier des conditions limites
! pour le problme d'lasticit fictif de "Mesh Update"
! Ces conditions limites sont de type dplacement interdit
! - Mesh Update 1 = Real 0
! - Mesh Update 2 = Real 0
! ou de type dplacement gal au dplacement de la paroi 
! - Mesh Update 1 = Equals Displacement 1
! - Mesh Update 2 = Equals Displacement 2

Boundary Condition 1
  Name = "PumpingBC"
  Target Boundaries = 1
  Velocity 1 = 0
  Velocity 2 = Equals Mesh Velocity 2
  Mesh Update 1 = Real 0
  Mesh Update 2 = Variable Time
       Real Procedure "./Motion" "Motion"
! La routine fortran Motion.f90 calcule le dplacement vertical
! de la Physical Line(1) 
! qui reprsente la variation volumique du ventricule
End


Boundary Condition 2
  Name = "OutletBC"
  Target Boundaries = 2
  Flux Integrate = Logical True
  Flow Force BC = True
! Condition de pression uniquement,  l outlet	
  Pressure 2 = Variable Time
    Real Procedure "ArteryOutlet" "OutletPres"
  Mesh Update 2 = Real 0
  1D Fluid Coupling = Logical True
End

Boundary Condition 3
  Name = "Symmetry3"
  Target Boundaries = 3
  Velocity 1 = 0 ! condition de glissement (vitesse horizontale nulle sur l axe)
  Mesh Update 1 = Real 0
End

Boundary Condition 4
  Name = "BC4"
  Target Boundaries = 4
  Velocity 1 = 0 ! condition de glissement (vitesse horizontale nulle sur l axe)
  Mesh Update 1 = Real 0
  Mesh Update 2 = Real 0
End

Boundary Condition 5
  Name = "BC5"
  Target Boundaries = 5
  Velocity 1 = 0 ! condition de glissement (vitesse horizontale nulle sur l axe)
  Mesh Update 1 = Real 0
End

Boundary Condition 6
  Name = "BC6"
  Target Boundaries(2) = 6 7 
  Mesh Update 1 = Real 0 ! Maillage fix au niveau des Physical Lines 6 et 7
  Mesh Update 2 = Real 0
End

Boundary Condition 7
  Name = "Wall"
  Target Boundaries = 8
  Displacement 1 = 0 ! Encastrement  la base de la paroi arterielle
  Displacement 2 = 0
End

Boundary Condition 8
  Name = "StructureEnd"
  Target Boundaries = 9
! Dplacement horizontal impos par le modle 1D 
!  l extrmit suprieure de la paroi artrielle 
  Displacement 1 = Variable Time
    Real Procedure "ArteryOutlet" "OutletdX"
  Displacement 2 = 0 
  1D Structure Coupling = Logical True
End

Boundary Condition 9 ! lateral side of the artery
  Name = "OuterWall"
! Surface libre (libre de contrainte) pour le problme d lasticit
  Target Boundaries = 10
!  Displacement 2 = 0 ! artere prise dans un bloc rigide
End

Boundary Condition 10 ! lieu de l interaction Fluide-Structure
  Name = "FSI"
  Target Boundaries = 11
  Velocity 1 = Equals Mesh Velocity 1
  Velocity 2 = Equals Mesh Velocity 2
  Mesh Update 1 = Equals Displacement 1
  Mesh Update 2 = Equals Displacement 2
  Force BC = Logical True
  Calculate Fluidic Force = Logical True
End

Boundary Condition 11
  Name = "Wall"
  Target Boundaries(3) = 12 13 14
  Mesh Update 1 = Real 0
  Mesh Update 2 = Real 0
  Velocity 1 = 0
  Velocity 2 = 0
End

Boundary Condition 12
  Name = "BC12"
  Target Boundaries = 15
  Mesh Update 1 = Real 0
  Velocity 1 = 0
  Velocity 2 = 0
End
